@@ -27,7 +27,7 @@
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: tokenizer.c,v 1.14 2003/07/08 17:08:49 kazuma-t Exp $
+ * $Id: tokenizer.c,v 1.5 2007/07/03 05:53:44 masayu-a Exp $
  */
 
 #include <string.h>
@@ -94,39 +94,36 @@ cha_tok_new(int lang, int encode)
 
     tok->lang = lang;
     tok->encode = encode;
-    tok->_is_malloced = 0;
-    tok->_char_type = tok->__static_char_type;
-    tok->_anno_type = tok->__static_anno_type;
     tok->anno = NULL;
 
     if (lang == CHASEN_LANG_JA) {
 	if (encode == CHASEN_ENCODE_EUCJP) {
-	    tok->_mblen = euc_mblen;
-	    tok->_char_type_parse = ja_char_type_parse;
-	    tok->_get_char_type = (ja_char_type_get)ja_euc_char_type;
+	    tok->mblen = euc_mblen;
+	    tok->char_type_parse = ja_char_type_parse;
+	    tok->get_char_type = (ja_char_type_get)ja_euc_char_type;
 	} else if (encode == CHASEN_ENCODE_SJIS) {
-	    tok->_mblen = sjis_mblen;
-	    tok->_char_type_parse = ja_char_type_parse;
-	    tok->_get_char_type = (ja_char_type_get)ja_sjis_char_type;
+	    tok->mblen = sjis_mblen;
+	    tok->char_type_parse = ja_char_type_parse;
+	    tok->get_char_type = (ja_char_type_get)ja_sjis_char_type;
 	} else if (encode == CHASEN_ENCODE_UTF8) {
-	    tok->_mblen = utf8_mblen;
-	    tok->_char_type_parse = ja_char_type_parse;
-	    tok->_get_char_type = (ja_char_type_get)ja_utf8_char_type;
+	    tok->mblen = utf8_mblen;
+	    tok->char_type_parse = ja_char_type_parse;
+	    tok->get_char_type = (ja_char_type_get)ja_utf8_char_type;
 	}
     } else if (lang == CHASEN_LANG_EN) {
 	if (encode == CHASEN_ENCODE_ISO8859) {
-	    tok->_mblen = iso8859_mblen;
-	    tok->_char_type_parse = en_char_type_parse;
-	    tok->_get_char_type = (en_char_type_get)en_char_type;
+	    tok->mblen = iso8859_mblen;
+	    tok->char_type_parse = en_char_type_parse;
+	    tok->get_char_type = (en_char_type_get)en_char_type;
 	} else if (encode == CHASEN_ENCODE_UTF8) {
-	    tok->_mblen = utf8_mblen;
-	    tok->_char_type_parse = en_char_type_parse;
-	    tok->_get_char_type = (en_char_type_get)en_char_type;
+	    tok->mblen = utf8_mblen;
+	    tok->char_type_parse = en_char_type_parse;
+	    tok->get_char_type = (en_char_type_get)en_char_type;
 	}
     } else {
-	tok->_mblen = iso8859_mblen;
-	tok->_char_type_parse = en_char_type_parse;
-	tok->_get_char_type = (en_char_type_get)en_char_type;
+	tok->mblen = iso8859_mblen;
+	tok->char_type_parse = en_char_type_parse;
+	tok->get_char_type = (en_char_type_get)en_char_type;
     }
 
     return tok;
@@ -138,77 +135,52 @@ cha_tok_new(int lang, int encode)
 void
 cha_tok_delete(chasen_tok_t *tok)
 {
-    if (tok->_is_malloced) {
-	cha_free(tok->_char_type);
-	cha_free(tok->_anno_type);
-    }
     cha_free(tok);
 }
 
-/*
- * This function parses string str with len bytes.
- */
 int
-cha_tok_parse(chasen_tok_t *tok, unsigned char *str, int len)
+cha_tok_parse(chasen_tok_t *tok, unsigned char *str, char *type, int len,
+	      int *anno_no)
 {
     int cursor, head;
     int state, state0;
     anno_info *anno = NULL;
+    int no;
 
-    tok->string = str;
-    tok->string_len = len;
-
-    if (len > sizeof(tok->__static_char_type) / sizeof(int)) {
-	tok->_char_type = cha_malloc(sizeof(int) * len);
-	tok->_anno_type = cha_malloc(sizeof(int) * len);
-	tok->_is_malloced = 1;
+    if (anno_no != NULL && (no = is_anno(tok, str, len)) >= 0) {
+	anno = &(tok->anno[no]);
+	*anno_no = no;
+	for (cursor = anno->len1;
+	     cursor < len;
+	     cursor += tok->mblen(str + cursor, len - cursor)) {
+	    if (is_anno2(anno, str, cursor))
+		break;
+	}
+	type[0] = cursor; /* 2007/06/26 YD */
+	return cursor;
     }
 
-    memset(tok->_char_type, 0, sizeof(int) * len);
-    memset(tok->_anno_type, 0, sizeof(int) * len);
-
     state0 = state = 0; /* NOSTATE */
     for (cursor = head = 0; cursor < len;
-	 cursor += tok->_mblen(str + cursor, len - cursor)) {
-	if (state0 < 0) { /* in annotation */
-	    if (is_anno2(anno, str, cursor)) {
-		state0 = 0; /* end of annotation */
-	    } else {
-		continue; /* skip annotation */
-	    }
-	} 
-
-	state = is_anno(tok, str + cursor, len - cursor);
-	if (state < 0) {
-	    anno = &(tok->anno[-state]);
-	    tok->_anno_type[cursor] = -state;
+	 cursor += tok->mblen(str + cursor, len - cursor)) {
+	if (anno_no != NULL &&
+	    is_anno(tok, str + cursor, len - cursor) >= 0) {
+   		type[head] = cursor - head; /* 2007/06/26 YD */
+	    return cursor;
 	} else {
-	    state = tok->_get_char_type(tok, str + cursor, len - cursor);
-	    state = tok->_char_type_parse(tok, state, &state0, cursor);
+	    state = tok->get_char_type(tok, str + cursor, len - cursor);
+	    state = tok->char_type_parse(tok, state, &state0, cursor);
 	}
 
 	if (state != state0) {
-	    tok->_char_type[head] = cursor - head;
+	    type[head] = cursor - head;
 	    head = cursor;
 	}
 	state0 = state;
     }
-    tok->_char_type[head] = cursor - head;
+    type[head] = cursor - head;
 
-    return 1;
-}
-
-/*
- * This function returns the length in bytes of the multibyte character
- * on cursor in the parsed string.
- *
- * If the character is `\0', it returns 1.
- */
-int
-cha_tok_mblen_on_cursor(chasen_tok_t *tok, int cursor)
-{
-    return tok->_mblen(tok->string + cursor,
-		       tok->string_len - cursor);
+    return cursor;
 }
 
 /*
@@ -220,19 +192,7 @@ cha_tok_mblen_on_cursor(chasen_tok_t *tok, int cursor)
 int
 cha_tok_mblen(chasen_tok_t *tok, unsigned char *str, int len)
 {
-    return tok->_mblen(str, len);
-}
-
-/*
- * This function returns the length in bytes of the substring
- * including same kind of characters.
- *
- * In the middle of substring, it returns 0.
- */
-int
-cha_tok_char_type_len(chasen_tok_t *tok, int cursor)
-{
-    return tok->_char_type[cursor];
+    return tok->mblen(str, len);
 }
 
 /*
@@ -244,37 +204,6 @@ cha_tok_set_annotation(chasen_tok_t *tok, anno_info *anno)
     tok->anno = anno;
 }
 
-/*
- * This function returns the type of annotation, if there is 
- * the begining of annotation on cursor.  The type is more than 0.
- *
- * And it returns less than 0 value, if the character on cursor is
- * white space.
- *
- * Otherwise it returns 0.
- */
-int
-cha_tok_anno_type(chasen_tok_t *tok, int cursor)
-{
-    return tok->_anno_type[cursor];
-}
-
-/*
- * This function is ad-hoc. You should not use it.
- *
- * But it is used in lib/parse.c(cha_parse_sentence).
- */
-int
-cha_tok_is_jisx0208_latin(chasen_tok_t *tok, int cursor, int len)
-{
-    if ((tok->lang == CHASEN_LANG_JA) &&
-	(tok->_get_char_type(tok, tok->string + cursor, len) == FULL_LATIN))
-	return 1;
-    else
-	return 0;
-
-}
-
 /*
  * private functions
  */
@@ -329,7 +258,7 @@ static int
 ja_char_type_parse(chasen_tok_t *tok, int state, int *state0, int cursor)
 {
     if (state == JA_SPACE) {
-	tok->_anno_type[cursor] = -1;
+	/* tok->anno_type[cursor] = 0; */ /* XXX */
     } else if ((state == HALF_LATIN) ||
 	       (state == FULL_LATIN)) {
 	; /* do nothing */
@@ -350,7 +279,7 @@ static int
 en_char_type_parse(chasen_tok_t *tok, int state, int *state0, int cursor)
 {
     if (state == EN_SPACE) {
-	tok->_anno_type[cursor] = -1;
+	/* tok->anno_type[cursor] = 0; */ /* XXX */
     } else if (state == EN_OTHER) {
 	*state0 = EN_NOSTATE;
     }
@@ -361,7 +290,7 @@ en_char_type_parse(chasen_tok_t *tok, int state, int *state0, int cursor)
 static enum ja_char_type
 ja_euc_char_type(chasen_tok_t *tok, unsigned char *str, int len)
 {
-    int mblen = tok->_mblen(str, len);
+    int mblen = tok->mblen(str, len);
 
     if (mblen == 1) {
 	if (isalpha(str[0])) {
@@ -393,7 +322,7 @@ ja_euc_char_type(chasen_tok_t *tok, unsigned char *str, int len)
 static enum ja_char_type
 ja_sjis_char_type(chasen_tok_t *tok, unsigned char *str, int len)
 {
-    int mblen = tok->_mblen(str, len);
+    int mblen = tok->mblen(str, len);
 
     if (mblen == 1) {
 	if (isalpha(str[0])) {
@@ -426,7 +355,7 @@ ja_sjis_char_type(chasen_tok_t *tok, unsigned char *str, int len)
 static enum ja_char_type
 ja_utf8_char_type(chasen_tok_t *tok, unsigned char *str, int len)
 {
-    int mblen = tok->_mblen(str, len);
+    int mblen = tok->mblen(str, len);
 
     if (mblen == 1) {
 	if (isalpha(str[0])) {
@@ -483,21 +412,30 @@ en_char_type(chasen_tok_t *tok, unsigned char *str, int len)
 static int
 is_anno(chasen_tok_t *tok, unsigned char *string, int len)
 {
-    int i;
+    int i, j;
     anno_info *anno = tok->anno;
 
-    if (anno == NULL) {
+    /* spaces are anno[0] (SPACE_POS) */
+    j = 0;
+    while (j < len && isspace(string[j]))
+        j++;
+    if (j) {
+	anno[0].len1 = j;
 	return 0;
     }
+
+    if (anno == NULL) {
+	return -1;
+    }
     for (i = 1; (anno[i].str1 != NULL); i++) {
 	if (len < anno[i].len1) {
 	    continue;
 	}
 	if (!memcmp(string, anno[i].str1, anno[i].len1)) {
-	    return -i;
+	    return i;
 	}
     }
-    return 0;
+    return -1;
 }
 
 static int
